import json
import sys
import configparser
import os
import re
import shutil
import subprocess
import xml.etree.ElementTree as ET

def runTest(log_dir, test_name):

    NfsName = getparams(dir,test_name,'NFSHOST')
    NfsPath = getparams(dir,test_name,'NFSPATH')
    VMDK = getparams(dir,test_name,'VMDK')
    primaryHost = getenvdata(log_dir,'esx','hostname')[0]
    TestDriver = getparams(dir,test_name,'TESTDRIVER')
    vmhba = getparams (dir,test_name,'VMHBA')
    vcserver = getparams (dir,test_name,'VCSERVER')
    Datastore = getparams (dir,test_name,'Datastore')
    DevName = getparams (dir,test_name,'DEVNAME')
    pythonpath = "/opt/vmware/toolchain/cayman_python-a7b24ae34668cc829e72c7f5317e9316a4286de2/lin64/bin/python3"
    FrameworkLauncherpath= "/sas67-cert/cert_stage1/TestLauncher.py"
    TestGroup1 = ["Storage::Stress::VMAddRemoveVirtualDisk","Storage::Stress::ResetRescan","Storage::Functional::Suspend_Resume","Storage::Functional::NoVM_CrashDump","Storage::Functional::NoVM_CreateVMFS","Storage::Functional::NoVM_DeviceStateChange","Storage::Functional::NoVM_DriverLoadUnload","Storage::Functional::NoVM_LoadESX_Reboot","Storage::Functional::NoVM_Reboot","Storage::Functional::NoVM_Reset"]
    TestGroup2 = ["Asyncdriver::Functional::Api_complaint","Asyncdriver::Functional::Stacksizecheck","Asyncdriver::Functional::Vib_VersionCheck","Asyncdriver::Functional::Stacksizecheck"]
    TestGroup3 = ["Storage::Stress::SVmotionWithSnapshot","Storage::Stress::NoVM_CreateImportVMDK"]
    TestGroup4 = ["Asyncdriver::Functional::UEFISecureboot_check"]
    TestGroup5 = ["Asyncdriver::Functional::4KnSupportCheck"]
    TestGroup6 = ["ESX::DDV::EILoading","ESX::DDV::EIRandomNoReload","ESX::DDV::StopNoReload"]
    if test_name in TestGroup1:
        TestCommand = ''+pythonpath+' '+FrameworkLauncherpath+' '+primaryHost+' --drivers '+TestDriver+' --vmdk RHEL65_64_8G --vmhbas '+vmhba+' --skip_vm_support'
    elif test_name in TestGroup2:
        TestCommand = ''+pythonpath+' '+FrameworkLauncherpath+' '+primaryHost+' --drivers '+TestDriver+' --vib_cfg=/sas67-cert/cert_stage1/resources/vib_config.json'
    elif test_name in TestGroup3:
        TestCommand = ''+pythonpath+' '+FrameworkLauncherpath+' '+primaryHost+' --drivers '+TestDriver+' --vmdk RHEL65_64_8G --vmhbas '+vmhba+' --vc_server '+vcserver+''
    elif test_name in TestGroup4:
        TestCommand = ''+pythonpath+' '+FrameworkLauncherpath+' '+primaryHost+' --drivers '+TestDriver+' --datastore '+Datastore+' --vib_cfg=/sas67-cert/cert_stage1/resources/vib_config.json'
    elif test_name in TestGroup5:
        TestCommand = ''+pythonpath+' '+FrameworkLauncherpath+' '+primaryHost+' --drivers '+TestDriver+' --devices '+DevName+' --vib_cfg=/sas67-cert/cert_stage1/resources/vib_config.json'
    elif test_name in TestGroup6:
        TestCommand = ''+pythonpath+' '+FrameworkLauncherpath+' '+primaryHost+' --drivers '+TestDriver+' --ddv_options cfgEILoadingDelay=200'
    else:
        TestCommand = ''+pythonpath+' '+FrameworkLauncherpath+' '+primaryHost+'  --drivers install --vib_cfg=/sas67-cert/cert_stage1/resources/vib_config.json'
    process = subprocess.Popen(''+TestCommand+' --test_ids '+test_name+' --skip_vm_support', cwd=log_dir,shell=True,stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    runlogfile = open(log_dir+'/run.log', 'w')
    runlogfile.write("Test")
    resultfile = open(log_dir+'/.result', 'w')
    testresult = False
    while True:
        nextline = process.stdout.readline().decode('utf-8')
        if nextline == '' and process.poll() is not None:
            break
        runlogfile.write(nextline)
        sys.stdout.write(nextline),
        sys.stdout.flush()
        if 'Setting logs folder:' in nextline:
            logs=nextline.split('Setting logs folder:')[1]
        if 'Test '+test_name+' PASSED' in nextline:
            testresult = True
        #runlogfile.write(nextline)
        #sys.stdout.write(nextline),
        #sys.stdout.flush()
    if testresult == True:
        resultfile.write('Passed')
    else:
        resultfile.write('Failed')
    runlogfile.close
    resultfile.close
    output = process.communicate()[0]
    exitCode = process.returncode
    #print(exitCode)
    tmplog = log_dir+"/logs"
    shutil.copytree(logs.strip(),tmplog)

def createjson(dir,test_name):
    vibmods = getparams(dir,test_name,'VIBMODS')
    vibversions = getparams(dir,test_name,'VIBVERSION')
    vibpaths = getparams(dir,test_name,'VIBPATH')
    acceptancelevel = getparams(dir,test_name,'ACCEPTANCELEVEL')
    rebootrequired = getparams(dir,test_name,'REBOOTREQUIRED')
    basevibversions = getparams(dir,test_name,'BASEVIBVERSION')
    basevibpaths = getparams(dir,test_name,'BASEVIBPATH')
    upgradevibversions = getparams(dir,test_name,'UPGRADEVIBVERSION')
    upgradevibpaths = getparams(dir,test_name,'UPGRADEVIBPATH')

    data= { 
        "vibmods": [vibmods],
        "vibversions": [vibversions],
        "vibpaths": [vibpaths],
        "acceptancelevel": acceptancelevel,
        "rebootrequired": rebootrequired,
        "basevibversions":[basevibversions],
        "basevibpaths": [basevibpaths] ,
        "upgradevibversions":[upgradevibversions] ,
        "upgradevibpaths":[upgradevibpaths]
        }
    jsonData = json.dumps(data)
    with open('/sas67-cert/cert_stage1/resources/vib_config.json', 'w') as f:
        json.dump(data, f)


def createconfig(log_dir):
    esxHostRootPwd = getenvdata(log_dir,'esx','password')[0]
    data= {
           "ESX": {"user":"root", "password": esxHostRootPwd},
           "VC" : {"user":"administrator@vsphere.local", "password": "Admin!23"}
        }
    jsonData = json.dumps(data)
    with open('/sas67-cert/cert_stage1/resources/credentials.json', 'w') as f:
        json.dump(data, f)

def WriteToXML(dir,log_dir,test_name):
    tree = ET.parse("/sas67-cert/cert_stage1/config.xml")
    root = tree.getroot()
    for child in root:
        for abc in child.iter('nfsserver'):
            abc.text=getparams(dir,test_name,'NFSHOST')
        for de in child.iter('nfsshare'):
            de.text=getparams(dir,test_name,'NFSPATH')
        for vm in child.iter('vm'):
            vm.set('VMDK',getparams(dir,test_name,'VMDK'))
        for server in child.iter('Server'):
            server.set('NAME',getenvdata(log_dir,'esx','hostname')[0])
        for host in child.iter('Host'):
            host.set('NAME',getenvdata(log_dir,'esx','hostname')[0])
    tree.write('/sas67-cert/cert_stage1/config.xml')
    #file = open('/sas67-cert/cert_stage1/config.xml','r')
    #print(file.read())

def getenvdata(dir, hosttype,key):
    with open('/runlist.json') as json_data:
        data = json.load(json_data)
        return [li[key] for li in data['environment']['hosts'] if li['hosttype'] == hosttype]

def getparams(dir,test_name,key):
    with open('/runlist.json') as json_data:
        data = json.load(json_data)
        #return  data['params'][key]
        for item in [li['params'] for li in data['params']['testParams'] if li['name'] == test_name]:
            if key in item:
               return item[key]
            else:
               item[key]="NA"
               return item[key] 


def main():
    subprocess.call('echo "trust machine * level 5" >> /usr/local/staf/bin/STAF.cfg', shell=True)
    subprocess.call('echo "trust user * level 5" >> /usr/local/staf/bin/STAF.cfg', shell=True)
    #file = open('/usr/local/staf/bin/STAF.cfg','r')
    #print(file.read())
    process1 = subprocess.Popen('export LD_LIBRARY_PATH=/usr/local/staf/lib;export PYTHONPATH=/usr/local/staf/lib/python31:/usr/local/staf/lib;/usr/local/staf/startSTAFProc.sh', cwd=sys.argv[2],shell=True,stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    dir = "/"
    test_name = sys.argv[1]
    createconfig(sys.argv[2])
    WriteToXML("/",sys.argv[2],sys.argv[1])
    if test_name.startswith('Asyncdriver'):
        createjson("/",sys.argv[1])
    runTest(sys.argv[2],sys.argv[1])

if __name__== '__main__':
    main()

